<?php

namespace CarDealerPress\Includes\Database;

use CarDealerPress\includes\functions\Logger;
use CarDealerPress\includes\helpers\helpers as helpers;
use CarDealerPress\Includes\Helpers\Option;
use CarDealerPress\Includes\Helpers\Remote_Requests;
use CarDealerPress\Includes\Helpers\Transients;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Company_Handler {
	use Logger;

	private static $blog;
	private static $wpdb;
	private Requests $db_requests;
	private $admin_options;

	public function __construct() {
		global $wpdb;
		self::$wpdb = $wpdb;
		self::$blog = get_current_blog_id();
		$this->set_option_variables();
	}

	public function set_option_variables() {
		$this->admin_options = Option::get_cdp_admin_options();
		$this->db_requests   = new Requests();
	}

	public function init_company_handler( $args = array() ) {
		$automall_run = isset( $args['automall_run'] ) ? '' : false;
		$company_id   = $args['company_id'] ?? helpers::company_id();
		$purge        = $args['purge'] ?? false;

		if ( $company_id ) {
			if ( ! get_transient( 'cdp_company_processing_' . self::$blog ) ) {
				set_transient( 'cdp_company_processing_' . self::$blog, 'Running', 60 );
				// Start by checking/adding additional dealers.
				$query_prepare        = 'select * from cdp_data where blog_id = %d and data_key = %s';
				$query_params_prepare = array( self::$blog, 'dealer' );
				$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
				$data_dealers         = self::$wpdb->get_results( $query, ARRAY_A );
				if ( ! empty( $data_dealers ) ) {
					// Loop through cdp_data dealers.
					foreach ( $data_dealers as $key => $data_dealer ) {
						$query_prepare        = 'select * from cdp_companies where blog_id = %d and dealer_id = %d';
						$query_params_prepare = array( self::$blog, $data_dealer['dealer_id'] );
						$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
						$dealer_check         = self::$wpdb->get_results( $query, ARRAY_A );
						// Add Dealer to companies table if not found.
						if ( empty( $dealer_check ) ) {
							$this->append_log( array( 'message' => 'CDP Message: Dealer ' . $data_dealer['dealer_id'] . ' missing from company | Adding to company table' ) );
							$data   = array(
								'blog_id'      => self::$blog,
								'company_id'   => $company_id,
								'dealer_id'    => $data_dealer['dealer_id'],
								'last_updated' => 0,
							);
							$format = array( '%d', '%d', '%d', '%d' );
							self::$wpdb->insert( 'cdp_companies', $data, $format );
						}
					}
					// Now loop through all companies entries and update if needed.
					$query_prepare        = 'select * from cdp_companies where blog_id = %d';
					$query_params_prepare = array( self::$blog );
					$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
					$dealers              = self::$wpdb->get_results( $query, ARRAY_A );

					if ( $dealers ) {
						foreach ( $dealers as $key => $dealer ) {
							$api_params = '/api/companies/' . $dealer['dealer_id'];
							if ( $purge ) {
								Remote_Requests::purge_url( $api_params );
							}
							$response      = Remote_Requests::http_get_file( $api_params );
							$response_code = wp_remote_retrieve_response_code( $response );
							if ( 200 === $response_code ) {
								$response_data = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
								// Update Company entry if new or update needed.
								$date_check = strtotime( $response_data->updated ) > intval( $dealer['last_updated'] );
								if ( $date_check || 0 === intval( $dealer['last_updated'] ) || $purge ) {
									$this->append_log( array( 'message' => 'CDP Message: Company Dealer ID ' . $dealer['dealer_id'] . ' manually updated' ) );
									( new Inserts() )->populate_companies_table(
										array(
											'data'       => $response_data,
											'type'       => 'update',
											'company_id' => $company_id,
										)
									);
									// Update Inventory with new data.
									if ( ! wp_next_scheduled( 'cdp_cron_inventory_update' ) ) {
										$this->append_log( array( 'message' => 'CDP Message: Company Inventory update backup. No Cron found' ) );
										( new Inventory_Handler() )->init_inventory_handler( $dealer['dealer_id'] );
									}
								}
							}
							// Remove dealer from companies if not found in cdp data.
							$query_prepare        = 'select * from cdp_data where blog_id = %d and dealer_id = %d and data_key = %s';
							$query_params_prepare = array( self::$blog, $dealer['dealer_id'], 'dealer' );
							$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
							$dealer_remove_check  = self::$wpdb->get_results( $query, ARRAY_A );
							if ( empty( $dealer_remove_check ) ) {
								$this->append_log( array( 'message' => 'CDP Message: Dealer ' . $dealer['dealer_id'] . ' being removed from companies as no longer found in CDP Data' ) );
								$where        = array( 'id' => $dealer['id'] );
								$where_format = array( '%d' );
								self::$wpdb->delete( 'cdp_companies', $where, $where_format );
							}
						}
						// Run Cron Job once if able.
						if ( wp_next_scheduled( 'cdp_cron_inventory_update' ) ) {
							$this->append_log( array( 'message' => 'CDP Message: Company Inventory update cron init.' ) );
							wp_schedule_single_event( time(), 'cdp_cron_inventory_update' );
						}
					}
					// Check if Primary Company has an automall and clean up new/remove dealers.
					if ( ! $automall_run ) {
						$new_automall_found   = false;
						$query_prepare        = 'select * from cdp_companies where blog_id = %d and company_id = %d and dealer_id = %d';
						$query_params_prepare = array( self::$blog, $company_id, $company_id );
						$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
						$dealer               = self::$wpdb->get_results( $query, ARRAY_A );
						if ( $dealer ) {
							$automalls = unserialize( $dealer[0]['automalls'] );
							if ( ! empty( $automalls ) ) {
								// Delete Dealers from CDP Data and Company if ID removed from automall.
								$query_prepare        = 'select * from cdp_data where blog_id = %d and data_key = %s and data_value = %s';
								$query_params_prepare = array( self::$blog, 'dealer', 'automall' );
								$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
								$data_automalls       = self::$wpdb->get_results( $query, ARRAY_A );
								if ( $data_automalls ) {
									foreach ( $data_automalls as $key => $data_automall ) {
										if ( ! in_array( intval( $data_automall['dealer_id'] ), $automalls, true ) ) {
											$this->append_log( array( 'message' => 'CDP Message: Automall Dealer (' . $data_automall['dealer_id'] . ') no longer found in Company Automall' ) );
											// Remove Company Value.
											$where        = array(
												'blog_id' => self::$blog,
												'company_id' => $company_id,
												'dealer_id' => $data_automall['dealer_id'],
											);
											$where_format = array( '%d', '%d', '%d' );
											self::$wpdb->delete( 'cdp_companies', $where, $where_format );
											// Remove from inentory - should already be gone, inventory cron runs hourly.
											self::$wpdb->delete( 'cdp_inventory_' . self::$blog, $where, $where_format );
											// Remove Data Value.
											$where        = array( 'data_id' => $data_automall['data_id'] );
											$where_format = array( '%d' );
											self::$wpdb->delete( 'cdp_data', $where, $where_format );
										}
									}
								}
								// Add/Update automall dealers in CDP Data.
								foreach ( $automalls as $key => $automall ) {
									// Check if automall dealer is already added to CDP data and update or add.
									$query_prepare        = 'select * from cdp_data where blog_id = %d and data_key = %s and dealer_id = %d';
									$query_params_prepare = array( self::$blog, 'dealer', $automall );
									$query                = self::$wpdb->prepare( $query_prepare, $query_params_prepare );
									$data_check           = self::$wpdb->get_results( $query, ARRAY_A );
									if ( empty( $data_check ) ) {
										// Add Dealer to CDP Data.
										$data        = array(
											'blog_id'    => self::$blog,
											'dealer_id'  => $automall,
											'data_key'   => 'dealer',
											'data_value' => 'automall',
										);
										$data_format = array( '%d', '%d', '%s', '%s' );
										self::$wpdb->insert( 'cdp_data', $data, $data_format );
										$new_automall_found = true;
									} else {
										// Update Dealer in CDP Data make sure data value is set to automall.
										// ** This could be due to a dealer that was manually added in the plugin being added as an automall dealer in VMS.
										$data         = array( 'data_value' => 'automall' );
										$data_format  = array( '%s' );
										$where        = array( 'data_id' => $data_check[0]['data_id'] );
										$where_format = array( '%d' );
										self::$wpdb->update( 'cdp_data', $data, $where, $data_format, $where_format );
									}
								}
								// Re-Run company handler if new a new automall company was added but automall check.
								if ( $new_automall_found ) {
									Transients::delete_cdp_company_processing( self::$blog );
									$this->init_company_handler( array( 'company_id' => $data['dealer_id'] ) );
								}
							} else {
								$this->append_log( array( 'message' => 'CDP Message: Company ID does not contain any automall data' ) );
							}
						}
					}
				}
			}
		}
	}

	public function init_dealer_refresh() {
		$query_results = true;
		$this->append_log( array( 'message' => 'CDP Message: Import: Dealer Manual Refresh' ) );
		delete_transient( 'cdp_company_check' );
		$query       = 'DELETE from cdp_companies where blog_id="' . self::$blog . '"';
		$sql_results = self::$wpdb->query( $query );
		if ( false !== $query_results ) {
			$this->company_check();
		}
	}

	public function check_for_company_update() {

		if ( false === ( get_transient( 'cdp_check_for_company_update_' . self::$blog ) ) ) {
			set_transient( 'cdp_check_for_company_update_' . self::$blog, 'Running', 60 );
			$companies = $this->db_requests->cdp_company_request(); // cdp_db_init()
			if ( $companies ) {
				foreach ( $companies as $key => $dealer ) {
					$response      = Remote_Requests::check_for_valid_company_id( $dealer['dealer_id'] );
					$response_code = wp_remote_retrieve_response_code( $response );
					if ( 200 === $response_code ) {
						$response_data = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
						// Update Company entry if new or update needed.
						$date_check = strtotime( $response_data->updated ) > intval( $dealer['last_updated'] );
						if ( $date_check ) {
							$this->append_log( array( 'message' => 'CDP Message: Company Dealer ID ' . $dealer['dealer_id'] . ' automatically updated' ) );
							( new Inserts() )->populate_companies_table(
								array(
									'data'       => $response_data,
									'type'       => 'update',
									'company_id' => $dealer['company_id'],
								)
							);
						}
					}
				}
			}
		}
	}
			// $query   = 'select * from cdp_companies where blog_id = "' . self::$blog . '" and company_id = "' . helpers::company_id() . '" AND dealer_id = "' . $this->company_id . '" ';
			// $results = self::$wpdb->get_results( $query, ARRAY_A );
			// if ( empty( $results ) ) {
			// $this->append_log( array( 'message' => 'CDP Message: Import: Adding new dealer ' . $this->company_id ) );
			// set_transient( 'cdp_company_check', 'Waiting 12 hours before next check.', 3600 * 12 );
			// $this->init_dealer_call();
			// $this->add_cdp_data_dealer();
			// $this->add_manual_dealers();
			// $this->remove_invalid_vehicles();
			// $this->company_duplicate_check();
			// } else {
			// $this->append_log( array( 'message' => 'CDP Message: Company Checked. Next Check in 12 Hours' ) );
			// $api_params    = '/api/companies/' . $this->company_id;
			// $response      = Remote_Requests::http_get_file( $api_params );
			// $response_code = wp_remote_retrieve_response_code( $response );
			// if ( 200 === $response_code ) {
			// $this->dealer = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
			// if ( isset( $this->dealer->updated ) ) {
			// set_transient( 'cdp_company_check', 'Waiting 12 hours before next check.', 3600 * 12 );
			// if ( $results[0]['last_updated'] < strtotime( $this->dealer->updated ) ) {
			// $this->append_log( array( 'message' => 'CDP Message: Import: Updating dealer ' . $this->company_id ) );
			// self::$wpdb->query( 'DELETE from cdp_companies where blog_id="' . self::$blog . '"' );
			// $this->init_dealer_call();
			// $this->add_cdp_data_dealer();
			// $this->add_manual_dealers();
			// $this->remove_invalid_vehicles();
			// $this->company_duplicate_check();
			// }
			// }
			// }
			// }

	public function company_duplicate_check() {

		$query       = 'DELETE FROM cdp_companies WHERE id in ( SELECT max_id from ( SELECT max(id) as max_id FROM cdp_companies WHERE dealer_id IN ( SELECT dealer_id FROM (SELECT dealer_id, count(*) AS duplicates FROM cdp_companies WHERE company_id = ' . $this->company_id . ' AND blog_id = ' . self::$blog . ' GROUP BY dealer_id ) AS z where z.duplicates > 1 ) GROUP BY id ) AS a )';
		$sql_results = self::$wpdb->query( $query );
		if ( $sql_results > 0 ) {
			$this->append_log( array( 'message' => 'CDP Message: Company Duplicates Removed' ) );
		}
	}

	public function init_dealer_call() {
		$api_params    = '/api/companies/' . $this->company_id;
		$response      = Remote_Requests::http_get_file( $api_params );
		$response_code = wp_remote_retrieve_response_code( $response );
		if ( 200 === $response_code ) {
			$this->dealer = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
			if ( $this->dealer ) {
				$this->populate_company_table();
				if ( isset( $this->dealer->automall_ids ) ) {
					$dealers = $this->dealer->automall_ids;
					foreach ( $dealers as $dealer ) {
						if ( $dealer !== $this->company_id ) {
							$api_params    = '/api/companies/' . $dealer;
							$response      = Remote_Requests::http_get_file( $api_params );
							$response_code = wp_remote_retrieve_response_code( $response );
							if ( 200 === $response_code ) {
								$this->dealer = isset( $response['body'] ) ? json_decode( wp_kses_data( $response['body'] ) ) : array();
								$this->populate_company_table();
							}
						}
					}
				}
			}
		}
	}

	public function add_cdp_data_dealer() {
		// Checks and creates entries to cdp_data from pre-set companies through an Auto Mall

		$query   = 'Select * from cdp_companies where blog_id = "' . self::$blog . '"';
		$dealers = self::$wpdb->get_results( $query, ARRAY_A );
		if ( ! empty( $dealers ) ) {
			foreach ( $dealers as $dealer ) {
				$query      = 'Select * from cdp_data where blog_id ="' . self::$blog . '" and dealer_id="' . $dealer['dealer_id'] . '" and data_key="dealer"';
				$data_check = self::$wpdb->get_results( $query, ARRAY_A );
				if ( empty( $data_check ) ) {
					$table  = 'cdp_data';
					$format = array( '%d', '%d', '%s', '%s' );
					// Insert Dealer
					$data = array(
						'blog_id'    => self::$blog,
						'dealer_id'  => $dealer['dealer_id'],
						'data_key'   => 'dealer',
						'data_value' => '',
					);
					self::$wpdb->insert( $table, $data, $format );
				}
			}
		}
	}

	public function remove_invalid_vehicles() {

		$query   = 'select dealer_id from cdp_companies where blog_id = "' . self::$blog . '" and company_id = "' . $this->company_id . '" ';
		$results = self::$wpdb->get_results( $query, ARRAY_A );
		$ids     = array();
		foreach ( $results as $key => $value ) {
			$ids[] = $value['dealer_id'];
		}
		if ( ! empty( $ids ) ) {
			$query = 'DELETE from cdp_inventory_' . self::$blog . ' where blog_id = "' . self::$blog . '" and company_id = "' . $this->company_id . '" and dealer_id not in (' . implode( ',', $ids ) . ')';
			self::$wpdb->query( $query );
		}
	}

	public function add_manual_dealers() {
		// Checks and creates entries to cdp_companies for manually added dealers

		$query   = 'Select * from cdp_data where blog_id = "' . self::$blog . '" and data_key = "dealer"';
		$dealers = self::$wpdb->get_results( $query, ARRAY_A );
		if ( ! empty( $dealers ) ) {
			foreach ( $dealers as $dealer ) {
				$query      = 'Select * from cdp_companies where blog_id ="' . self::$blog . '" and dealer_id="' . $dealer['dealer_id'] . '" and company_id="' . $this->company_id . '"';
				$data_check = self::$wpdb->get_results( $query, ARRAY_A );
				if ( empty( $data_check ) ) {
					$api_params    = '/api/companies/' . $dealer['dealer_id'];
					$response      = Remote_Requests::http_get_file( $api_params );
					$response_code = wp_remote_retrieve_response_code( $response );
					if ( 200 === $response_code ) {
						$this->dealer = isset( $response['body'] ) ? json_decode( $response['body'] ) : array();
						if ( $this->dealer ) {
							$this->populate_company_table();
						}
					}
				}
			}
		}
	}


}
